import { __decorate } from "tslib";
import { DatasourceDefaults, DatasourceEventHandler, FieldTypeEnum, logger, ResourceType, } from '@genesislcap/foundation-comms';
import { Connect } from '@genesislcap/foundation-comms';
import { attr, observable } from '@microsoft/fast-element';
import { FoundationElement } from '@microsoft/fast-foundation';
import { dateValueFormatter, dateTimeValueFormatter, getColumnType, getFilterByFieldType, } from '../grid-pro.definitions';
import { GridProRendererTypes } from '../grid-pro.types';
export class GenesisGridDatasourceElement extends DatasourceEventHandler(FoundationElement) {
    constructor() {
        super(...arguments);
        this.isSnapshot = false;
        this.maxRows = DatasourceDefaults.MAX_ROWS_250;
        this.maxView = DatasourceDefaults.MAX_VIEW_1000;
        this.movingView = false;
        this.reverse = false;
        this.isRequestServer = false;
    }
    /* Returns the `row-id` attribute, depending on the resource type.
     * @remarks Will favour the `row-id` attribute if set.
     * @internal
     */
    get rowId() {
        if (this.rowIdAttr)
            return this.rowIdAttr;
        if (this.isRequestServer)
            return DatasourceDefaults.REQUEST_SERVER_ROW_ID;
        return DatasourceDefaults.DATASERVER_ROW_ID;
    }
    /**
     * Returns whether the `row-id` attribute is the default one, depending on the resource type.
     * @internal
     */
    get hasDefaultRowId() {
        return (!this.rowIdAttr ||
            this.rowIdAttr === DatasourceDefaults.REQUEST_SERVER_ROW_ID ||
            this.rowIdAttr === DatasourceDefaults.DATASERVER_ROW_ID);
    }
    get defaultRowIdByResourceType() {
        return this.isRequestServer
            ? DatasourceDefaults.REQUEST_SERVER_ROW_ID
            : DatasourceDefaults.DATASERVER_ROW_ID;
    }
    datasourceStatusChanged(prev, next) {
        this.isRequestServer = next.type === ResourceType.REQUEST_SERVER;
    }
    get agGrid() {
        return this.parentElement;
    }
}
__decorate([
    Connect
], GenesisGridDatasourceElement.prototype, "connect", void 0);
__decorate([
    observable
], GenesisGridDatasourceElement.prototype, "deferredGridOptions", void 0);
__decorate([
    observable
], GenesisGridDatasourceElement.prototype, "deferredColumnStates", void 0);
__decorate([
    observable
], GenesisGridDatasourceElement.prototype, "rowDataMapper", void 0);
__decorate([
    attr
], GenesisGridDatasourceElement.prototype, "criteria", void 0);
__decorate([
    attr
], GenesisGridDatasourceElement.prototype, "fields", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'is-snapshot' })
], GenesisGridDatasourceElement.prototype, "isSnapshot", void 0);
__decorate([
    attr({ attribute: 'max-rows' })
], GenesisGridDatasourceElement.prototype, "maxRows", void 0);
__decorate([
    attr({ attribute: 'max-view' })
], GenesisGridDatasourceElement.prototype, "maxView", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'moving-view' })
], GenesisGridDatasourceElement.prototype, "movingView", void 0);
__decorate([
    attr({ attribute: 'order-by' })
], GenesisGridDatasourceElement.prototype, "orderBy", void 0);
__decorate([
    attr({ attribute: 'resource-name' })
], GenesisGridDatasourceElement.prototype, "resourceName", void 0);
__decorate([
    attr({ mode: 'boolean' })
], GenesisGridDatasourceElement.prototype, "reverse", void 0);
__decorate([
    attr({ attribute: 'row-id' })
], GenesisGridDatasourceElement.prototype, "rowIdAttr", void 0);
/**
 * The operation type for the {@link @genesislcap/grid-pro#GridProBaseDatasource.mapTransaction} method.
 * @internal
 */
export var OperationType;
(function (OperationType) {
    OperationType["Add"] = "add";
    OperationType["Update"] = "update";
    OperationType["Remove"] = "remove";
})(OperationType || (OperationType = {}));
export class GridProBaseDatasource extends GenesisGridDatasourceElement {
    constructor() {
        super(...arguments);
        this.dataSubWasLoggedOff = false;
        this.rowData = new Map();
        this.agTransaction = {
            add: [],
            remove: [],
            update: [],
        };
    }
    generateColumnDefsFromMetadata(fieldsMetadata, getFilterParamsByFieldType) {
        return fieldsMetadata === null || fieldsMetadata === void 0 ? void 0 : fieldsMetadata.flatMap((field) => {
            var _a;
            const colDefBase = {};
            colDefBase.filterParams = getFilterParamsByFieldType(field.type);
            switch (field.type) {
                case FieldTypeEnum.STRING:
                    colDefBase.cellEditor = GridProRendererTypes.stringEditor;
                    break;
                case FieldTypeEnum.NUMBER:
                    colDefBase.cellEditor = GridProRendererTypes.numberEditor;
                    break;
                case FieldTypeEnum.BOOLEAN:
                    if (this.agGrid.autoCellRendererByType) {
                        colDefBase.cellRenderer = GridProRendererTypes.boolean;
                        colDefBase.cellEditor = GridProRendererTypes.boolean;
                    }
                    break;
                case FieldTypeEnum.DATE:
                    colDefBase.valueFormatter = dateValueFormatter;
                    colDefBase.cellEditor = GridProRendererTypes.dateEditor;
                    break;
                case FieldTypeEnum.DATETIME:
                    colDefBase.valueFormatter = dateTimeValueFormatter;
                    colDefBase.cellEditor = GridProRendererTypes.dateEditor;
                    colDefBase.cellEditorParams = {
                        withTime: true,
                    };
                    break;
                case FieldTypeEnum.ENUM:
                    if (field.enumOptions) {
                        colDefBase.cellEditor = GridProRendererTypes.selectEditor;
                        colDefBase.cellEditorParams = {
                            values: field.enumOptions.map((option) => ({ value: option, label: option })),
                            valueField: 'value',
                            labelField: 'label',
                        };
                    }
                    else {
                        logger.warn(`No enumOptions found for field ${field.name}. Default cell editor applied.`);
                    }
                    break;
                default:
                    break;
            }
            const colDefExtras = (_a = this.agGrid.gridApi) === null || _a === void 0 ? void 0 : _a.getColumnDef(field.name);
            if (this.fields && !this.fields.includes(field.name)) {
                return [];
            }
            return Object.assign(Object.assign({ field: field.name, type: getColumnType(field.type), filter: getFilterByFieldType(field.type) }, colDefBase), colDefExtras);
        });
    }
    handleStreamInserts(insertedRows) {
        insertedRows === null || insertedRows === void 0 ? void 0 : insertedRows.forEach((insertData) => {
            if (this.rowData.has(insertData[this.rowId]) || this.dataSubWasLoggedOff) {
                const rowToBeUpdated = this.rowData.get(insertData[this.rowId]);
                const updatedRow = Object.assign(Object.assign({}, rowToBeUpdated), insertData);
                this.agTransaction.update.push(updatedRow);
            }
            else {
                this.agTransaction.add.push(insertData);
                this.agTransaction.addIndex = 0;
            }
            this.rowData.set(insertData[this.rowId], insertData);
        });
    }
    handleStreamDeletes(deletedRows) {
        deletedRows === null || deletedRows === void 0 ? void 0 : deletedRows.forEach((deleteData) => {
            if (this.hasDefaultRowId || this.isRequestServer) {
                this.agTransaction.remove.push({ [this.rowId]: deleteData[this.rowId] });
            }
            else {
                const rowToBeDeleted = Object.values(Object.fromEntries(this.rowData)).find((obj) => obj[this.defaultRowIdByResourceType] === deleteData[this.defaultRowIdByResourceType]);
                this.agTransaction.remove.push({ [this.rowId]: rowToBeDeleted[this.rowId] });
            }
            this.rowData.delete(deleteData[this.rowId]);
        });
    }
    handleStreamUpdates(updatedRows) {
        updatedRows === null || updatedRows === void 0 ? void 0 : updatedRows.forEach((updateData) => {
            let updatedRow;
            if (this.hasDefaultRowId || this.isRequestServer) {
                const rowToBeUpdated = this.rowData.get(updateData[this.rowId]);
                updatedRow = Object.assign(Object.assign({}, rowToBeUpdated), updateData);
                this.agTransaction.update.push(updatedRow);
            }
            else {
                const rowToBeUpdated = Object.values(Object.fromEntries(this.rowData)).find((obj) => obj[this.defaultRowIdByResourceType] === updateData[this.defaultRowIdByResourceType]);
                updatedRow = Object.assign(Object.assign({}, rowToBeUpdated), updateData);
                this.agTransaction.update.push(updatedRow);
            }
            this.rowData.set(updateData[this.rowId], updatedRow);
        });
    }
    applyAllAgTransactions() {
        var _a, _b, _c;
        if (!this.agTransaction.add.length &&
            !this.agTransaction.update.length &&
            !this.agTransaction.remove.length) {
            this.agGrid.gridApi.hideOverlay();
            if (this.rowData.size === 0) {
                this.agGrid.gridApi.showNoRowsOverlay();
            }
            return;
        }
        if (((_a = this.agTransaction.add) === null || _a === void 0 ? void 0 : _a.length) > 0) {
            this.applyMappedAgTransaction({ add: this.agTransaction.add, addIndex: this.agGrid.addIndex }, OperationType.Add, this.agGrid.asyncAdd);
        }
        if (((_b = this.agTransaction.remove) === null || _b === void 0 ? void 0 : _b.length) > 0) {
            this.applyMappedAgTransaction({ remove: this.agTransaction.remove }, OperationType.Remove, this.agGrid.asyncRemove);
        }
        if (((_c = this.agTransaction.update) === null || _c === void 0 ? void 0 : _c.length) > 0) {
            this.applyMappedAgTransaction({ update: this.agTransaction.update }, OperationType.Update, this.agGrid.asyncUpdate);
        }
    }
    flashAddedCells(rowNodes) {
        if (this.agGrid.enableRowFlashing && (rowNodes === null || rowNodes === void 0 ? void 0 : rowNodes.length)) {
            this.agGrid.gridApi.flashCells({ rowNodes });
        }
    }
    applyMappedAgTransaction(agTransaction, operation, isAsync) {
        if (isAsync) {
            this.applyTransactionAsync(this.mapTransaction(agTransaction, [operation]), (changedNodes) => {
                this.flashAddedCells(changedNodes.add);
            });
        }
        else {
            const changedNodes = this.applyTransaction(this.mapTransaction(agTransaction, [operation]));
            this.flashAddedCells(changedNodes.add);
        }
    }
    applyTransaction(agTransaction) {
        throw new Error('Method not implemented.');
    }
    applyTransactionAsync(agTransaction, callback) {
        throw new Error('Method not implemented.');
    }
    /**
     * Maps the transaction data to the row data mapper function, if it exists.
     * @param transaction - The transaction data to be mapped.
     * @param operations - The operations to be mapped. Can be 'add', 'update' or 'remove'.
     * @returns The mapped transaction (if the row data mapper function exists), or the original transaction.
     */
    mapTransaction(transaction, operations = Object.values(OperationType)) {
        if (!this.rowDataMapper) {
            return transaction;
        }
        const mappedTransaction = {};
        if (operations.includes(OperationType.Add)) {
            mappedTransaction.add = this.rowDataMapper(transaction.add, this.rowData);
        }
        if (operations.includes(OperationType.Remove)) {
            mappedTransaction.remove = this.rowDataMapper(transaction.remove, this.rowData);
        }
        if (operations.includes(OperationType.Update)) {
            mappedTransaction.update = this.rowDataMapper(transaction.update, this.rowData);
        }
        return mappedTransaction;
    }
}
export class StreamBaseDatasource {
    constructor() {
        this.rowData = new Map();
    }
}
